#!/usr/bin/perl -w

##############################################################
#
# This program reads the TeX source files for the Whitman Calculus
# book and produces an HTML version.
#
# Run the program like so:
#
#    tex_to_html.pl inputfile outputdirectory
#
# it is assumed that the inputfile is of the form
#
# inputdirectory/mainfile.tex
#
# where mainfile.tex reads in files from the inputdirectory.
# Also, it is assumed that the image files are in the subdirectory
# inputdirectory/html_figures
#
# The structure of this program is:
#
# 1.  Check that the command line arguments are sensible
# 2a. Set up global variables (see below for a discussion)
# 2b. Read the auxiliary files and populate the global variables
# 3.  Read through the inputfile, locating the files which contain the contents
#       of the book (typically, one file per chapter).
# 4.  Process each file as you find it (see below for a discussion)
# 5.  Write the auxiliary files
# 6.  Done!    
#
# The main principle is that we have to process a succession of nested
# "environments" or "components."  For example, a section of a book
# may contain definitions, theorems, figures, etc.  We use a regular
# expression to locate these items (which is easy, because each has a
# beginning and ending tag), and then call a function to process the
# content of that component/environment.
#
# Even more basic than the above example is the book itself: it
# contains chapters (and some material before the first chapter and
# after the last chapter).
#
# Each chapter contains sections.
#
# The above observations explain the structure of the code:
#
# At the top level we scan the main file, locating the chapters.  Each
# chapter is handed to the processchapter() function.  That function,
# among other things, finds the sections and hands them to the
# processsection() function.  That function, among other things, finds
# the definitions, theorems, figures, etc, and hands them other
# functions for procesing.
#
# The hope is that the general structure of this program can be easily adapted to
# process other books: one only needs to modify the regular expressions that
# detect the various environments and parameters.
#
# Comments on global variables (step 2):
#
# Many components of a documents are referenced by a label: the author
# can assign labels to the chapters, sections, definitions, examples,
# theorems, etc, and later refer to the component by label.  As the
# components are processed they are assigned numbers (for example:
# Definition 5.2.3).  We use global variables to keep track of
# assigned numbers (so that we can increment the counter and assign
# the correct number to the next definition), and also to remember
# what number was assigned to a particular definition (more precisely,
# what number is associated to the label associated to the
# definition).
#
# These global variables are saved in files in the output directory,
# similar to the way LaTeX uses auxiliary files.

#
# Coments on step 4:
#
# Each processenvironment() function typically does the following things,
#     but not necessarily in this order.:
#
# Determine the "name" of the component (eg, section title)
# Increment the counter for that component
# Determine if there is a label or the component. If there is:
#     Remember the data associated to the label
#     Save the content of the component in a knowl (if appropriate), and
#        remember the name of the knowl file.
# Convert TeX markup to HTML
# Locate any sub-components and send them to their own processing function.
#
# Figure files are in html and are copied verbatim to the appropriate
# place in the final document.
#
#############################################################

######################
#
# Start of program.
#
# The first dozen lines check the command line inputs, and if they look
# reasonable, set up the main input file, input directory, and output directory
#
use Getopt::Std;

my $multivariable = 1;
my $latetranscendentals = 0;
my $exercises_link = 1;
getopts("msle",\%opts);

if ($opts{'m'}) {
  $multivariable = 1;
}
if ($opts{'s'}) {
  $multivariable = 0;
}
if ($opts{'l'}) {
   $latetranscendentals = 1;
}
if ($opts{'e'}) {
   $exercises_link = 0;
}

if($#ARGV != 1){
     usage();
     exit 0;
} # ARGV if

my $inputfile=$ARGV[0] || die "no input file specified\n";
unless(-e $inputfile) { die "the input file $inputfile does not exist\n"}

my $inputdirectory;
my $maininputdirectory;
my $basename;
if($inputfile =~ /(.+)\/(.+)\.tex/) {
   $inputdirectory = $1;
   $maininputdirectory = $inputdirectory;
   $basename = $2;
}
else { die "input file should be in a subdirectory which also contains the other input files\n"}
unless(-d $inputdirectory) { die "the input directory $inputdirectory does not exist\n"}

my $outdir=$ARGV[1] || die "no output directory specified\n";
unless(-d $outdir) { die "the output directory $outdir does not exist\n"}

my $realchapnum = 0;
#
# GLOBAL VARIABLES
#
# set up the global variables, and populate them with material from
# the previous run
#

#
# navigation menu
#
my $navcontents;  # the navigation menu (table of contents) that 
                  # appears on every page; this is the version from 
                  # the previous run.
my $navfilename=$outdir."/nav.txt";  # file where we save the navigation menu
$navcontents = getfromfile($navfilename);

my $tocnav="";  # the navigation menu -- the new version created during this run
$tocnav .= qq|<div id="menu_expand" style="display:none"><a href="javascript:toggle_sidebar()">Expand menu</a></div>\n<div id="sidebar" style="display:block">\n<h2 id="menu_collapse"><a href='javascript:toggle_sidebar()'>Collapse menu</a></h2>|;

#
# global variables for references, titles of sections and chapters, files containing
# chapters and sections, etc.
#
my %xrefn=();      # used analogously to LaTeX \ref
my %idtohtmlfile=(); # DRG A hash to record which file various anchors are in, 
                     # using the id from \thmrdef, \secrdef, \exrdef, \figrdef.

my %chaptitle=();    # we read these in from a file, then update them, then write to the
my %sectitle=();     # file for use the next time.  We always use these hashes when 
my %chaphtmlfile=(); # we need the title of a section or chapter, or the html file 
my %sechtmlfile=();  # of the section or chapter.

my %figtitle=();     # the "title" of a figure is its numbering, in the form chap.sec.num

my $hashesfile=$outdir."/hashes.txt";
populateglobalvariables($hashesfile);

#
#  Create CSS file
#
# writecssfile();

#
# read through the  main file, locating the chapters and sections

open MAINFILE,  "<$inputfile"  or die "Can't open $inputfile: $! \n";

while($line=<MAINFILE>) {  # loop through the main file, locating the lines that read in chapters

if(!$latetranscendentals && $line =~ /\\iflatetranscendentals/) { 
   while($tmpline = <MAINFILE>) {
## print("LATE SKIPPING: $tmpline \n");
       if ( ($tmpline =~ /^\\filatetranscendentals/) ||
	    ($tmpline =~ /^\\elselatetranscendentals/)) { last; }
       next;
   }
}
if($latetranscendentals && $line =~ /\\elselatetranscendentals/) { 
   while($tmpline = <MAINFILE>) {
#print("EARLY SKIPPING: $tmpline \n");
       if ( ($tmpline =~ /^\\filatetranscendentals/)) { last; }
       next;
   }
}


## DRG fi -> fimultivariable
if(!$multivariable && $line =~ /\\ifmultivariable/) {  # skip the multivariable chapters
   while($tmpline = <MAINFILE>) {
##print("MULTI SKIPPING: $tmpline \n");
       unless($tmpline =~ /^\\fimultivariable/) { next }
       last
   }
}

## DRG Only at line beginning, so we can comment them out.
if($line =~ /^\\input +chapter([0-9bc]+)/) {
  $chapnum=$1;
  $realchapnum++;
  print("\n------------------\n");
#  print("found chapter ", $chapnum,"-->",$realchapnum,"\n");
  processchapter($chapnum,$realchapnum)
}

## 
if($line =~ /^\\setinputdirectory\{([a-z0-9_\/]+)\}/) {
  $inputdirectory=$1;
  print "inputdirectory now $inputdirectory\n";
}

}

#
# Now we save the global variables.  Probably these should be put into a subroutine.


$tocnav .= qq|\n</div>\n|;

open NAVFILE, ">$outdir/nav.txt" or die "Couldn't open file: $!";

print NAVFILE $tocnav;

close NAVFILE;;

open INDEXFILE, ">$outdir/index.html" or die "Couldn't open file: $!";
open TITLEFILE, "$maininputdirectory/book_title.txt";
$book_title = <TITLEFILE>;
close TITLEFILE;
chomp $book_title;
print "book title: $book_title\n";
print INDEXFILE header("",$book_title);
print INDEXFILE $navcontents;
print INDEXFILE titlepage();
print INDEXFILE footer();
close INDEXFILE;

open NAMESFILE, ">$outdir/hashes.txt" or die "Couldn't open file: $!";
while (($key, $value) = each(%xrefn)){
     print NAMESFILE "xrefn:".$key."=>".$value."\n";
}
print NAMESFILE "\n";
while (($key, $value) = each(%chaptitle)){
     print NAMESFILE "chaptitle:".$key."=>".$value."\n";
}
print NAMESFILE "\n";
while (($key, $value) = each(%sectitle)){
     print NAMESFILE "sectitle:".$key."=>".$value."\n";
}
print NAMESFILE "\n";
while (($key, $value) = each(%figtitle)){
     print NAMESFILE "figtitle:".$key."=>".$value."\n";
}
print NAMESFILE "\n";
while (($key, $value) = each(%sechtmlfile)){
     print NAMESFILE "sechtmlfile:".$key."=>".$value."\n";
}
print NAMESFILE "\n";
while (($key, $value) = each(%chaphtmlfile)){
     print NAMESFILE "chaphtmlfile:".$key."=>".$value."\n";
}
print NAMESFILE "\n";
while (($key, $value) = each(%idtohtmlfile)){
  if (defined($value)) {
     print NAMESFILE "idtohtmlfile:".$key;
     print NAMESFILE "=>".$value."\n";
  }
}
#print NAMESFILE "\n";


close NAMESFILE;

exit 0;
## DRG: saving die for errors.
## die();


###########################################################
##
##sub processchapter {
##
##my $chapnum=shift;
##
##
##$chapcontents="";
##
##my $inputfile = $inputdirectory."/chapter".$chapnum.".tex";
##open INFILE, $inputfile  or die "Couldn't open file: $inputfile  $!";
##
##print "processing chapter $chapnum\n";
##
##my $iniflate=0;
##my $inelselate=0;
##my $firstchap=1;
##my $chaphtmlfile;
##
#### DRG: The late transcendentals construction can't be processed a line
#### at a time, so new code here to do it. Also 
#### else -> elselatetranscendentals and fi -> filatetranscendentals
##while($line = <INFILE>) {
##  if($line =~ /^\\elselatetranscendentals/) { 
##    $iniflate = 0; 
###    print "Starting else part\n";
##  }
##  elsif($line =~ /^\\filatetranscendentals/) { 
##    $iniflate = 0; 
###    print "End of late trans\n";
##  }
##  elsif($line =~ /^\\iflatetranscendentals/) { 
##    $iniflate = 1; 
###    print "Starting late transcendentals\n";
##  }
##  elsif ($iniflate) { 
###   print "Ignoring late trans: $line\n"; 
##    next; 
##  }
##  elsif($line =~ /^\\chapter{([^\}]+)}/)  {
##       $chaptitle=$1;
##       print "\n\nfound chapter $chapnum: $chaptitle\n\n";
##       $chaptitle =~ s/\\titlebreak/ /;
##       $chaptitle{$chapnum} = $chaptitle;
##       $chaphtmlfile{$chapnum}="chapter".$chapnum.".html";
##    open CHAPFILE, ">$outdir/$chaphtmlfile{$chapnum}" or die "Couldn't open file: $!";
##       print CHAPFILE header("chapter",$chaptitle{$chapnum},$chapnum);
##       $chapcontents .= qq|\n<div id="main">\n\n<div id="content">\n\n|;
#### DRG Don't want the chaptitle twice.
###       $chapcontents .= "<h1>".$chaptitle{$chapnum}."</h1>\n\n<p>\n";
##       $chapcontents .= "<p>\n";
##       $tocnav .= qq|\n<h2 class="link"><a href="$chaphtmlfile{$chapnum}">|.chapsecnumrewrite($chapnum).qq| $chaptitle{$chapnum}</a></h2>\n\n<ul class="list">\n|;
##       print "\n$tocnav\n";
##   }
##
##   elsif($line =~ /^\\input +section([0-9]{2}.[0-9]{2})/) {
##
##       $secnum=$1;
##       processsection($secnum)
##
##}
##
##   elsif($line =~ m/^\s*$/) { $chapcontents .= "</p>\n<p>\n" }
##  else {$chapcontents .=  $line;}
####   else {  # the line above, and the following lines are not really correct, because most of
####           # the pattern matching shold be done on large blocks of text, not one line at a time.
####           # but since the chapters are typically just a small amount of plain text,
####           # probably doesn't matter. 
####       $line = latexconvert1($line);  # convert common LaTeX commands to html
####       $line = whitmanconvert1($line);  #delete extraneous latex
####       $line = whitmanconvert2($line, "");  #process environments
####       $line = latexconvert2($line);  #  mostly italics and bold face
####       $line = latexconvert3($line);  # convert LaTeX non-english letters to html
######       $chapcontents .=  whitmanconvert($line);
####       $chapcontents .=  $line;
####   }
##
##}
##
##$chapcontents  = latexconvert1($chapcontents);  # convert common LaTeX commands to html
##$chapcontents  = whitmanconvert1($chapcontents);  #delete extraneous latex
##$chapcontents  = whitmanconvert2($chapcontents, "");  #process environments
##$chapcontents  = latexconvert2($chapcontents);  #  mostly italics and bold face
##$chapcontents  = latexconvert3($chapcontents);  # convert LaTeX non-english letters to html
##$tocnav .= qq|</ul>\n|;
##
##print CHAPFILE $navcontents;
##
##$chapcontents .=  qq|\n</div>\n</div>\n</body>\n|;
##print CHAPFILE $chapcontents;
##close CHAPFILE;
##
##}
##
###########################################################

sub processchapter {

my $chapnum=shift;
my $actualchapnum=shift;
my $actualsecnum=0;
if ($actualchapnum<10) { $actualchapnum = "0".$actualchapnum; }
$chapcontents="";
my $chaptoc="";
my $inputfile = $inputdirectory."/chapter".$chapnum.".tex";
open INFILE, $inputfile  or die "Couldn't open file: $inputfile  $!";

print "processing chapter $chapnum-->$actualchapnum\n";

my $iniflate=0;
my $inelselate=0;
my $firstchap=1;
my $chaphtmlfile;
my $chapheader="";
while($line = <INFILE>) {
  if($line =~ /^\\elselatetranscendentals/) { 
    $inelselate = 1;
    $iniflate = 0; 
#    print "Starting else part\n";
  }
  elsif($line =~ /^\\filatetranscendentals/) { 
    $iniflate = 0;
    $inelselate = 0; 
#    print "End of late trans\n";
  }
  elsif($line =~ /^\\iflatetranscendentals/) { 
    $iniflate = 1; 
#    print "Starting late transcendentals\n";
  }
  elsif (!$latetranscendentals && $iniflate) { 
#   print "Ignoring late trans: $line\n"; 
    next; 
  }
  elsif ($latetranscendentals && $inelselate) { 
#   print "Ignoring late trans: $line\n"; 
    next; 
  } elsif($line =~ /^\\chapter\{([^\}]+)}/)  {
    $chaptitle=$1;
    print "\nfound chapter $chapnum: $chaptitle\n\n";
    $chaptitle =~ s/\\titlebreak/ /;
    $chaptitle = latexconvert3($chaptitle);
    $chaptitle{$actualchapnum} = $chaptitle;
    $chaphtmlfile{$actualchapnum}="chapter".$actualchapnum.".html";
    open CHAPFILE, ">$outdir/$chaphtmlfile{$actualchapnum}" or die "Couldn't open file: $!";
    my $print_chapnum = $actualchapnum * 1;
    print CHAPFILE header("chapter","$print_chapnum. ".$chaptitle{$actualchapnum},$actualchapnum);
    $tocnav .= qq|\n<h2 class="link">|;
    $tocnav .= qq|<img id="chapter_|;
    $tocnav .= $actualchapnum;
    $tocnav .= qq|_button" src="expand_submenu.png"|;
    $tocnav .= qq| onclick='toggle("chapter_|;
    $tocnav .= $actualchapnum;
    $tocnav .= qq|_contents")'>|;
    $tocnav .= qq|<a href="$chaphtmlfile{$actualchapnum}">|.chapsecnumrewrite($actualchapnum).qq| $chaptitle{$actualchapnum}</a></h2>\n\n<ul class="list" id="chapter_|;
    $tocnav .= $actualchapnum;
    $tocnav .= qq|_contents">\n|;
#       print "\n$tocnav\n";
 } elsif($line =~ /^\\input +section([0-9a-z]+.[0-9]{2})/) {
   $secnum=$1;
   $actualsecnum++;
   $chaptoc .= processsection($secnum,$actualchapnum,$actualsecnum);
 } else {$chapcontents .=  $line;}
##   else {  # the line above, and the following lines are not really correct, because most of
##           # the pattern matching shold be done on large blocks of text, not one line at a time.
##           # but since the chapters are typically just a small amount of plain text,
##           # probably doesn't matter. 
##       $line = latexconvert1($line);  # convert common LaTeX commands to html
##       $line = whitmanconvert1($line);  #delete extraneous latex
##       $line = whitmanconvert2($line, "");  #process environments
##       $line = latexconvert2($line);  #  mostly italics and bold face
##       $line = latexconvert3($line);  # convert LaTeX non-english letters to html
####       $chapcontents .=  whitmanconvert($line);
##       $chapcontents .=  $line;
##   }

}
$figurecount=0;
if ($chapcontents =~ /\\chaprdef\{([^\{\}]+)}/s) {
  my $chapid=$1;
  $chapid =~ s/[ \t\n]+/ /g;
  $chapcontents =~ s/\\chaprdef\{([^\}]*)}/\n<p><a id=\"$chapid\"><\/a>\n/;
  $idtohtmlfile{$chapid} = $chaphtmlfile{$chapnum};
  $xrefn{$chapid} = chapsecnumrewrite($chapnum);
}

$chapcontents  = latexconvert1($chapcontents);  # convert common LaTeX commands to html
$chapcontents  = latexconvert2($chapcontents);  #  mostly italics and bold face
$chapcontents  = latexconvert3($chapcontents);  # convert LaTeX non-english letters to html
$chapcontents  = whitmanconvert1($chapcontents);  #delete extraneous latex
$chapcontents  = whitmanconvert2($chapcontents, "$chapnum.00");  #process environments
$tocnav .= qq|</ul>\n|;

print CHAPFILE $navcontents;
print CHAPFILE qq|\n<div id="main">\n|;
print CHAPFILE nextsectionnav($actualchapnum.".00");
print CHAPFILE qq|\n<div id="content">\n<p>\n|;
print CHAPFILE "<p>\n";
print CHAPFILE $chapcontents;
print CHAPFILE $chaptoc;
print CHAPFILE  qq|\n</div>\n</div>\n</body>\n|;
close CHAPFILE;
}

###########################################################

sub processsection {

my $secnumfilename=shift;
my $actualchapnum=shift;
my $actualsecnum=shift;
my $secnum_print = $actualsecnum;
if ($actualsecnum<10) { $actualsecnum = "0".$actualsecnum;}
my $secnum = "$actualchapnum.$actualsecnum";
my $chaptocentry="";
       $sechtmlfile{$secnum}="section".$secnum.".html";
       my $secfile="section".$secnumfilename.".tex";
       my $inputfile = $inputdirectory."/".$secfile;
print("The input file is: ",$inputfile, "\n");
       my $thesection;
     {
       open SECFILE, $inputfile  or die "Couldn't open file: $!";
       local $/=undef;
       $thesection = <SECFILE>;
       close SECFILE;
     }

# global counters used for the sequential figure numbers, etc
$examplecount=0;
$figurecount=0;
$theoremcount=0;
## DRG
$equationcount=0;
if($thesection =~/section\{([^\}]*)}\s*{([^\}]*)}\s*{([^\}]*)}/s) {
   $sectitle = $1;
   $altsectitle = $3;
   $sectitle = latexconvert3($sectitle);
}
else { $sectitle = "MISSING SECTION TITLE";
       print("Missing section title in section $secnum \n")
}

       $sectitle =~ s/\\titlebreak/ /;
       $sectitle{$secnum}=$sectitle;
       my $sectionanchor = "";
       if($thesection =~ /\\secrdef\{([^\{\}]+)}/s) {
         my $secid=$1;
         $secid =~ s/[ \t\n]+/ /g;
## DRG Replace \secrdef with html anchor.
	 $thesection =~ s/\\secrdef\{([^\}]*)}//;
## Save sectionanchor for later, so the "<"s don't get clobbered by
## latexconvert1.
         $sectionanchor="<p><a id=\"$secid\"></a>";
         $idtohtmlfile{$secid} = $sechtmlfile{$secnum};
         $xrefn{$secid} = chapsecnumrewrite($secnum);
       }

## DRG page references
       while ($thesection =~ /\\pagerdef\{([^\{\}]+)}/s) {
         my $pageid=$1;
         $pageid =~ s/[ \t\n]+/ /g;
         $pageanchor="<a id=\"$pageid\"></a>";
	 $thesection =~ s/\\pagerdef\{([^\}]*)}/$pageanchor/;
         $idtohtmlfile{$pageid} = $sechtmlfile{$secnum};
         $xrefn{$pageid} = chapsecnumrewrite($secnum);
       }

    open SECFILE, ">$outdir/$sechtmlfile{$secnum}" or die "Couldn't open file: $!";
       $tocnav .= qq|<li><a href="$sechtmlfile{$secnum}">|."$secnum_print. $sectitle{$secnum}".qq|</a></li>\n|;
       $chaptocentry = qq|\n<h3><a href="$sechtmlfile{$secnum}">|."$secnum_print. $sectitle{$secnum}".qq|</a></h3>\n|;
if ($thesection =~ /\\exercises/) {
  $exercise_section = 1;
} else {
  $exercise_section = 0;
}
if ($altsectitle =~ /\S/) {
  print SECFILE header("section",chapsecnumrewrite($secnum)." ".$sectitle{$secnum},$actualchapnum,$secnum,$exercise_section,$altsectitle);
} else {
  print SECFILE header("section",chapsecnumrewrite($secnum)." ".$sectitle{$secnum},$actualchapnum,$secnum,$exercise_section);
}
print SECFILE $navcontents;
$thesection = latexconvert1($thesection);  # convert common LaTeX commands to html
##if ($secnum =~ /04\.08/) {
##  open (junk,">junk");
##  print junk $thesection;
##  close junk;
##  }
$thesection = latexconvert2($thesection);  #  mostly italics and bold face
$thesection = latexconvert3($thesection);  # convert LaTeX non-english letters to html
$thesection = whitmanconvert1($thesection);  #delete extraneous latex
$thesection = whitmanconvert2($thesection, $secnum);  #process environments
print SECFILE "\n<div id=\"main\">\n";
print SECFILE nextsectionnav($secnum);
print SECFILE "\n<div id=\"content\">\n<p>\n";
## DRG Put in anchor.
print SECFILE "$sectionanchor\n";
print SECFILE $thesection;
print SECFILE "\n</div>\n";
print SECFILE "\n</div>\n";
close(SECFILE);
return($chaptocentry);
}

###########################################################
sub populateglobalvariables {
  my $hashesfile = shift;
  open NAMESFILE,  "<$hashesfile"  or warn "Need to run program again: $! \n";
  while ($line = <NAMESFILE>) {
    ##DRG Need the "?".
    ##     if($line =~ /(.+):(.+)+=>(.+)$/) {
    if ($line =~ /(.+?):(.+)+=>(.+)$/) {
      if ($1 eq "xrefn") {
	$xrefn{$2}=$3
      } elsif ($1 eq "chaptitle") {
	$chaptitle{$2}=$3
      } elsif ($1 eq "sectitle") {
	$sectitle{$2}=$3
      } elsif ($1 eq "chaphtmlfile") {
	$chaphtmlfile{$2}=$3
      } elsif ($1 eq "sechtmlfile") {
	$sechtmlfile{$2}=$3
      } elsif ($1 eq "figtitle") {
	$figtitle{$2}=$3
      } elsif ($1 eq "idtohtmlfile") {
	$idtohtmlfile{$2}=$3
      }
    }
  }
  close NAMESFILE;
}
###########################################################

sub getfromfile {   # this should be merged with the getfilecontents() function that
                    # is used to get the html figures

my $filename=shift;

  local $/=undef;
  open THEFILE,  "<$filename"  or warn "Does not exist $filename: $! \n";
  $filecontents = <THEFILE>;
#  print("Read $filename contents\n");
  close THEFILE;

return($filecontents);

}
###########################################################

sub globalcount {

my $type=shift;
my $ans;

if($type eq "exercise") {

   ++$exercisecount;
   $ans=$exercisecount;

}

elsif($type eq "example") {

   ++$examplecount;
   $ans=$examplecount;

}

elsif($type eq "figure") {

   ++$figurecount;
   $ans=$figurecount;

}

elsif($type eq "theorem") {

   ++$theoremcount;
   $ans=$theoremcount;

}

elsif($type eq "equation") {

   ++$equationcount;
   $ans=$equationcount;

}

else {

   $ans="XX"

}

return("".$ans)

}

###########################################################

sub latexconvert1 {
my $thetext=shift;

# remove TeX comments
$thetext =~ s/^\%.*//g;  # delete everything on the line after %
$thetext =~ s/\n\%.*//g;  # delete everything on the line after %
$thetext =~ s/([^\\])\%.*/$1 /g;  # delete everything on the line after %

$thetext =~ s/\\\%/\%/g;  # a literal percent
$thetext =~ s/\\pct/\\\%/g;  # a literal percent
$thetext =~ s/\\\&/&amp;/g;  # ampersand
$thetext =~ s/\\amp/\\\&/g;  # ampersand
$thetext =~ s/\\cents/&cent;/g;
$thetext =~ s/\\ / /g;  # a literal space
$thetext =~ s/\\hfill//g;
$thetext =~ s/\\break//g;

$thetext =~ s/</< /sg;   # don't want \lt to look like the start of a tag
## DRG Except when we do:
#$thetext =~ s/<\s+=/<=/g;
#$thetext =~ s/<\s+</<</g;
#$thetext =~ s/< iostream/&lt;iostream/g;
$thetext =~ s/< br>/<br>/g;
$thetext =~ s/< a href=/<a href=/g;
$thetext =~ s/< a id=/<a id=/g;
$thetext =~ s/< a knowl=/<a knowl=/g;
$thetext =~ s/ < \/a>/<\/a>/g;
$thetext =~ s/< \/a>/<\/a>/g;
$thetext =~ s/< b>/<b>/g;
$thetext =~ s/< b style=/<b style=/g;
$thetext =~ s/< \/b>/<\/b>/g;
$thetext =~ s/< img/<img/g;
$thetext =~ s/< br/<br/g;
$thetext =~ s/< form/<form/g;
$thetext =~ s/< \/form>/<\/form>/g;
$thetext =~ s/< script/<script/g;
$thetext =~ s/< \/script>/<\/script>/g;
$thetext =~ s/< table/<table/g;
$thetext =~ s/< \/table>/<\/table>/g;
$thetext =~ s/< \/tr>/<\/tr>/g;
$thetext =~ s/< \/td>/<\/td>/g;
$thetext =~ s/< \/th>/<\/th>/g;
$thetext =~ s/< em>/<em>/g;
$thetext =~ s/< \/em>/<\/em>/g;
$thetext =~ s/< tr /<tr /g;
$thetext =~ s/< tr>/<tr>/g;
$thetext =~ s/< h2>/<h2>/g;
$thetext =~ s/< td /<td /g;
$thetext =~ s/< th /<th /g;
$thetext =~ s/< td>/<td>/g;
$thetext =~ s/< th>/<th>/g;
$thetext =~ s/< p(.*?)>/<p$1>/g;
$thetext =~ s/< \/p>/<\/p>/g;
$thetext =~ s/< i>/<i>/g;
$thetext =~ s/< \/i>/<\/i>/g;
$thetext =~ s/< div(.*?)>/<div$1>/g;
$thetext =~ s/< \/div>/<\/div>/g;
$thetext =~ s/< center>/<center>/g;
$thetext =~ s/< \/center>/<\/center>/g;
$thetext =~ s/< code/<code/g;
$thetext =~ s/< \/code>/<\/code>/g;
$thetext =~ s/< blockquote>/<blockquote>/g;
$thetext =~ s/< \/blockquote>/<\/blockquote>/g;

$thetext =~ s/\\vskip *-*[0-9]+ *pt//g;
$thetext =~ s/\\vskip *-*[0-9\.]+ *in//g;
$thetext =~ s/\\nobreak//g;
$thetext =~ s/\\dosupereject\b//g;
$thetext =~ s/\\ssk\b//g;
$thetext =~ s/\\msk\b//g;
$thetext =~ s/\\bsk\b//g;
## DRG
$thetext =~ s/\\par\b/<\/p><p>/g;

$thetext =~ s/\\dots/&hellip;/g;
$thetext =~ s/\\noindent//g;
$thetext =~ s/\\thinspace//g;
$thetext =~ s/\\smallskip//g;

$thetext =~ s/\\relax\b//g;
$thetext =~ s/\\lefthead\{(.*?)}/<h2>{$1}<\/h2>/g;

$thetext =~ s/\`\`/"/g;
## DRG Screws up the second derivative. Seems to look ok as is.
##$thetext =~ s/\'\'/"/g;
#convert TeX paragraph breaks
$thetext =~ s/\n\n/\n<\/p><p>\n/sg;
$thetext =~ s/\r\n\r\n/\n<\/p><p>\n/sg;

return($thetext)
}

###########################################################
sub latexconvert2 {  # mostly italics and bold face

my $thetext=shift;


#$thetext =~ s/\\item\{}//g;
$thetext =~ s/\\item\{}/<\/p><p>/g;
#if ($thetext =~ /\\item{\\tt/) {
#  print "*************************** found tt\n";
#}
$thetext =~ s/\\item\{\\tt\s+([^\}]*?)}/<\/p><p>\n<code> $1<\/code>/g;
$thetext =~ s/\\item\{([^\}]+)}/<\/p><p> $1 /g;
$thetext =~ s/\\itemitem\{}//g;
$thetext =~ s/\\itemitem\{\\txtbold[ \t\n]+([^\}]+)}/<\/p><p><b> $1<\/b> /g;
$thetext =~ s/\\itemitem\{([^\}]+)}/<\/p><p> $1 /g;
$thetext =~ s/{\\dfont[ \t\n]+([^\}]*)\\\/}/<b>$1<\/b>/g;
$thetext =~ s/{\\dfont[ \t\n]+([^\}]*)}/<b>$1<\/b>/g;
# Use \em, not \it
#$thetext =~ s/{\\it[ \t\n]+([^\}]*)\\\/}/<em>$1<\/em>/g;
#$thetext =~ s/{\\it[ \t\n]+([^\}]*)}/<em>$1<\/em>/g;
#$thetext =~ s/{\\bf[ \t\n]+([^\}]*)\\\/}/<b>$1<\/b>/g;
#$thetext =~ s/{\\bf[ \t\n]+([^\}]*)}/<b>$1<\/b>/g;
$thetext =~ s/{\\tt[ \t\n]+([^\}]*)}/<code>$1<\/code>/g;
$thetext =~ s/{\\txtbold[ \t\n]+([^\}]*)\\\/}/<b>$1<\/b>/g;
$thetext =~ s/{\\txtbold[ \t\n]+([^\}]*)}/<b>$1<\/b>/g;
$thetext =~ s/{\\rm[ \t\n]*([^\}]*)}/$1/g;
$thetext =~ s/{\\sc[ \t\n]+([^\}]*)\\\/}/<b>$1<\/b>/g;
$thetext =~ s/{\\sc[ \t\n]+([^\}]*)}/<b>$1<\/b>/g;
$thetext =~ s/{\\subsecfont[ \t\n]+([^\}]*)}/<b>$1<\/b>/g;
$thetext =~ s/\\dotdivide/<\/p>\$\$\\bullet\\quad\\bullet\\quad\\bullet\$\$<p>/g;
$thetext =~ s/\\textRed(.*?)\\textBlack/<font color='red'>$1<\/font>/sg;

##$thetext =~ s/\emph{[ \t\n]+([^\}]*)}/<em>$1<\/em>/g;
$thetext =~ s/{\\em[ \t\n]+([^\}]*)\\\/}/<em>$1<\/em>/g;
$thetext =~ s/{\\em[ \t\n]+([^\}]*)}/<em>$1<\/em>/g;

return($thetext)
}
###########################################################
sub latexconvert3 {  # codes for non-english letters

my $thetext=shift;

$thetext =~ s/\\'([a-z]{1})/&$1acute;/g;
$thetext =~ s/\\'\{([a-z]{1})}/&$1acute;/g;

$thetext =~ s/\\`([a-z]{1})/&$1grave;/g;
$thetext =~ s/\\`\{([a-z]{1})}/&$1grave;/g;

$thetext =~ s/\\"([a-z]{1})/&$1uml;/g;
$thetext =~ s/\\"\{([a-z]{1})}/&$1uml;/g;
$thetext =~ s/\\umlaut ([a-z]{1})/&$1uml;/g;

$thetext =~ s/\\~([a-z]{1})/&$1tilde;/g;
$thetext =~ s/\\~\{([a-z]{1})}/&$1tilde;/g;

$thetext =~ s/\\\^([a-z]{1})/&$1circ;/g;
$thetext =~ s/\\\^\{([a-z]{1})}/&$1circ;/g;

$thetext =~ s/---/&mdash;/g;
$thetext =~ s/--/&ndash;/g;
return($thetext)
}

###########################################################
sub whitmanconvert1 {

my $thetext=shift;
if(!$latetranscendentals) {
  $thetext =~ s/\\iflatetranscendentals((.(?!iflatetranscendentals))*?)\\elselatetranscendentals(.*?)\\filatetranscendentals\b/$3/sg;
  $thetext =~ s/\\iflatetranscendentals((.(?!iflatetranscendentals))*?)\\filatetranscendentals\b//sg;
}
else {  
   $thetext =~ s/\\iflatetranscendentals((.(?!iflatetranscendentals))*?)\\elselatetranscendentals(.*?)\\filatetranscendentals\b/$1/sg;
   $thetext =~ s/\\iflatetranscendentals((.(?!iflatetranscendentals))*?)\\filatetranscendentals\b/$1/sg;
}

## DRG: remove ~ here.
# delete the code that is only for TeX
## $thetext =~ s/\\texonly.+?\\endtexonly/\n\n/sg;
$thetext =~ s/\\texonly.+?\\endtexonly\s*\n//sg;
#$thetext =~ s/\~/ /g;
# Convert ~ to space between non-whitespace characters. Some uses of ~
# have already been taken care of at this point.
$thetext =~ s/([\S])\~([\S])/$1 $2/g;
# delete the markers for html only text
$thetext =~ s/\\htmlonly.*?\n//g;
$thetext =~ s/\\endhtmlonly.*?\n//g;

# these probably should be moved to the place where \section, etc, is processed
$thetext =~ s/\\section\{([^\}]*)}[ \t\n]*{}[ \t\n]*{([^\}]*)}//;
$thetext =~ s/\\section\{([^\}]*)}[ \t\n]*{([^\}]*)}[ \t\n]*{([^\}]*)}//;
$thetext =~ s/\\section\{([^\}]*)} *{([^\}]*)}//;
## DRG:
## $thetext =~ s/\\secrdef{([^\}]*)}//;
$thetext =~ s/\\index\{([^\}]*)}//g;
$thetext =~ s/\\twocol//g;
$thetext =~ s/\\endtwocol//g;


# the following doesn't work right, because doubleitem is really a table
# but we can fix it be making \doubleitem two spans of fixed width.
$thetext =~ s/\\beginlist[ \t\n]+/<\/p><ul><p>\n/g;  # should set a class here
##$thetext =~ s/\\doubleitem\b/<li>/g;
$thetext =~ s/[ \t\n]+\\endlist\b/\n<\/p><\/ul><p>/g;
$thetext =~ s/\\doubleitem\s*{(.*)}\s*{(.*)}\s*{(.*)}\s*{(.*)}\s*$/<table style=\"width:100%\"<tr><td style=\"width:50%;text-align:left\">$1&nbsp;$2<\/td><td style=\"width:50%;text-align:left\">$3&nbsp;$4<\/td><\/tr><\/table>/g;
$thetext =~ s/[ \t\n]+\\extract\b/\n<blockquote>/g;
$thetext =~ s/[ \t\n]+\\endextract\b/\n<\/blockquote>/g;
# the \items need to be done properly
#$thetext =~ s/\\item{}//g;
#$thetext =~ s/\\item{([^\}]+)}/<\/p><p> $1 /g;
#$thetext =~ s/\\itemitem{}//g;
#$thetext =~ s/\\itemitem{([^\}]+)}/<\/p><p> $1 /g;

$thetext =~ s/\\cd\|(.*?)\|/qq|<code>|.fixcode($1).qq|<\/code>|/seg;
#$thetext =~ s/\\vb\|0\|/qq|<code>0<\/code>|/seg;
$thetext =~ s/\\vb\|(.*?)\|/qq|<code>|.fixcode($1).qq|<\/code>|/seg;
$thetext =~ s/\\vb\/(.*?)\//qq|<code>|.fixcode($1).qq|<\/code>|/seg;
$thetext =~ s/\\vglue[ \-0-9]+pt//g;
$thetext =~ s/\\goodbreak//g;
return($thetext)

}

###############
sub whitmanconvert2 {

my $thetext=shift;
my $chapsecnum=shift;

$chapsecnum=($chapsecnum || "");

$chapsecnum =~ /([0-9]+).([0-9]+)/;
my ($chapnum,$secnum)=($1,$2);

$chapnum = ($chapnum || "cchapnum");
$secnum = ($secnum || "ssecnum");

$chapnum =~ s/^0*//;
$secnum =~ s/^0*//;
my $secnumber=$chapnum.".".$secnum;

#
# We process the theorem-like environments.  These are handled by
# processtheorem(type,secnumber,content)
#
# Since we want to number theorems, lemmas, remarks, and examples in one
# sequentially numbered list, we have to write one regular expression that
# matches any of them.
#
# But we still need to process proof separately from the statement being proved.
#

$thetext =~ s/\\(defn|remark|example|thm|lem|cor|prop)\b *(.*?)\\end(def|remark|example|thmnoproof|thmlaterproof|thm|lem|prop)/processtheorem($1,$chapsecnum,$2)/seg;

##DRG \eqrdefs in displays. The resulting anchor has to be outside the
## display, and we don't want the number to be a link.

##$thetext =~ s/\$\$([^\$]*)\\eqrdef{([^}]+)}\\eqno{\(\\xrefn{[^}]+}\)}([^\$]*)\$\$/processdisplay($1,$chapsecnum,$2,$3)/seg;
$thetext =~ s/\$\$([^\$]*)\\eqrdef\{([^}]+)}\(\\xrefn\{[^}]+}\)([^\$]*)\$\$/processdisplay($1,$chapsecnum,$2,$3)/seg;

## DRG: Change qed to endproof, handle detachedproof.
# process the proofs
## $thetext =~ s/\\proof\b[ \t\n]*(.*?)\\qed\b/processproof($1).qq|<div class="qed">\$\\square\$<\/div>\n|/seg;
$thetext =~ s/\\proof\b[ \t\n]*(.*?)\\endproof\b/processproof($1).qq|<div class="qed">\$\\square\$<\/div>\n|/seg;
$thetext =~ s/\\detachedproof\{(.*?)}\n(.*?)\\endproof\b/processdetachedproof($1,$2).qq|<div class="qed">\$\\square\$<\/div>\n|/seg;
#
# Now we process the figures
#
$thetext =~ s/\\setbox\\scratch.*?\\begincode\b[ \t\n]*(.*?)\\endcode.*?\\endtikzpicture}/processcode($1)/seg;
$thetext =~ s/\\null\\vbox\{\\ninepoint\\tikzpicture.*?listcolor\]\s+{(.*?)};\s+\\endtikzpicture}/processcolorlist($1)/seg;
# if ($thetext =~ /\\null\\vbox{\\ninepoint\\tikzpicture.*?listcolor\]\s+{(.*?)};\s+\\endtikzpicture}/s) {
#    print "*************************************** found colorlist\n";
# }
$thetext =~ s/\\figure *\n(.*?)\n\\endfigure/processfigure($chapsecnum,$1)/seg;

# the above line handles graphics in a figure environment, but there are "bare"
# graphics which are not in such an environment.  The next line does those.
$thetext =~ s/\\htmlfigure\s*{(.+?)}.*?\n/"<\/p><p><div class='figure'>".getfilecontents("html_figures",$1)."\n<\/div><\/p><p>"/ge;

## DRG Knowls.
$thetext =~ s/\\htmlknowl\s*{([^\}]+?)}\{([^\}]+?)}\s*\n/<a knowl=\"$1\">$2<\/a>\n/g;

## DRG--remove tablelayout stuff
## if($tablelayout) {
##  $thetext =~ s/\\htmlfigure{(.+?)}/"<table class='figure'>\n<tr>\n".getfilecontents("html_figures",$1)."\n<\/tr>\n<\/table><!-- figure -->\n<p>\n"/sge;
## }
## else {   # this needs to be updated: \htmlonly no longer required 
##  $thetext =~ s/\\htmlonly\n(.*?)\\htmlfigure{(.+?)}\n(.*?)\\endhtmlonly/$1."\n".getfilecontents("html_figures",$2)."\n".$3."\n<\/p><p>"/sge;
## }

#
# Now the Exercise section
#

$thetext =~ s/\\exercises\*?\s*(.*?)\\endexercises/qq|<\/p>\n<a id="exercises"><\/a><h2 class="exercises">Exercises $secnumber<\/h2>\n<p>|.processexercises($chapsecnum,$1).qq|\n|/seg;

## DRG In case some xrefn has no ~, and because ~ may occur elsewhere,
## separate how they are handled--do all ~ chars at once elsewhere.
## $thetext =~ s/\~\\xrefn{([^\{\}]+)}/"&nbsp;".xrefn($1)/sge;
## $thetext =~ s/\\xrefn{([^\{\}]+)}/xrefn($1)/sge;

## DRG Make references links
$thetext =~ s/\\xrefn\{([^\{\}]+)}/"\<a href=\"".$idtohtmlfile{normalizeref($1)}."#".normalizeref($1)."\"\>".xrefn($1)."\<\/a\>"/sge;

# the following doesn't work right, because doubleitem is really a table
# but we can fix it be making \doubleitem two spans of fixed width.
$thetext =~ s/\\beginlist\b/<ul>/g;  # should set a class here
#$thetext =~ s/\\doubleitem\b/<li>/g;
$thetext =~ s/\\doubleitem\s*{(.*)}\s*{(.*)}\s*{(.*)}\s*{(.*)}\s*/<table style=\"width:100%\"><tr style=\"width:100%\"><td style=\"width:50%;text-align:left\">$1&nbsp;$2<\/td><td style=\"width:50%;text-align:left\">$3&nbsp;$4<\/td><\/tr><\/table>\n/g;
$thetext =~ s/\\endlist\b/<\/ul>/g;

# the \items need to be done properly
# $thetext =~ s/\\item{}//g;
# $thetext =~ s/\\item{([^\}]+)}/ $1 /g;

return($thetext)

}

###############

sub processexercises {

my $secnumber = shift;
my $thetext = shift;
$exercisecount=0;

$thetext =~ s/\\exercise\b\s*(.*?)\\endexercise/processexercise($secnumber,$1)/seg;

return($thetext);

}

#################

sub processexercise {  #singular, not plural

my $chapsecnum = shift;
my $theexercise = shift;
$chapsecnum =~ /([0-9]+).([0-9]+)/;
my ($chapnum,$secnum)=($1,$2);

$chapnum = ($chapnum || "cchapnum");
$secnum = ($secnum || "ssecnum");

$chapnum =~ s/^0*//;
$secnum =~ s/^0*//;
my $secnumber=$chapnum.".".$secnum;

if(
$theexercise =~ s/(.*?)\n*\\answer *(.*?)\n*\\endanswer/qq|<\/p>\n<p class="exercise"><b>Ex |.$secnumber.".".globalcount("exercise").qq|<\/b>\n$1\n(<a knowl="" class="internal" value="$2">answer<\/a>)\n|/egs
  ) { } # nothing to do, because it was done in the conditional
else {
  if (
      $theexercise =~ s/(.*?)\n*\\hint *(.*?)\n*\\endanswer/qq|<\/p>\n<p class="exercise"><b>Ex |.$secnumber.".".globalcount("exercise").qq|<\/b>\n$1\n(<a knowl="" class="internal" value="$2">hint<\/a>)\n|/egs
     ) { } # nothing to do, because it was done in the conditional

# the above takes care of exercises with answers.  Still have to take care of those without answers.
# This construction could have been handled with an "or"

  else{
    $theexercise = qq|<\/p>\n<p class="exercise"><b>Ex |.$secnumber.".".globalcount("exercise")."<\/b>\n".$theexercise;
  }
}
if($theexercise =~ /\\exrdef\{([^\{\}]+)}/s) {
       $exid=$1;
       $exid =~ s/[ \t\n]+/ /g;
## DRG
##       $xrefn{$exid} = $secnumber.".".$exercisecount;
       $xrefn{$exid} = $exercisecount;
## DRG Replace \\exrdef by html anchor.
	 $theexercise =~ s/\\exrdef\{([^\}]*)}/\<a id="$exid"\>\<\/a\>/s;
         $idtohtmlfile{$exid} = $sechtmlfile{$chapsecnum};
       }

## $theexercise =~ s/\\exrdef{([^\}]*)}//g;

$theexercise =~ s/\\solution\s*(.*?)\\endsolution//gs;

return($theexercise);

}

###############

sub processtheorem {

my $thetype=shift;
my $chapsecnum=shift;
my $thetheorem = shift;
my $theproof = (shift || "");
my $thmid="";

$chapsecnum =~ /([0-9]+).([0-9]+)/;
my ($chapnum,$secnum)=($1,$2);

$chapnum = ($chapnum || "cchapnum");
$secnum = ($secnum || "ssecnum");

$chapnum =~ s/^0*//;
$secnum =~ s/^0*//;
my $secnumber=$chapnum.".".$secnum;

my ($type,$Type);

#(defn|remark|example|thm|lem|cor|prop)

if($thetype eq "thm") { $type="theorem"; $Type="Theorem"}
elsif($thetype eq "lem") { $type="theorem"; $Type="Lemma"}
elsif($thetype eq "cor") { $type="theorem"; $Type="Corollary"}
elsif($thetype eq "defn") { $type="definition"; $Type="Definition"}
elsif($thetype eq "remark") { $type="remark"; $Type="Remark"}
elsif($thetype eq "example") { $type="example"; $Type="Example"}
elsif($thetype eq "prop") { $type="theorem"; $Type="Proposition"}

# print("PROcessing a THEorem\n");

my $thmnumber=globalcount("theorem");

if($thetheorem =~ /\\thmrdef\{([^\{\}]+)}/s) {
       $thmid=$1;
       $thmid =~ s/[ \t\n]+/ /g;
       $xrefn{$thmid} = $secnumber.".".$thmnumber;
## DRG Replace thmrdef by html anchor.
       $thetheorem =~ s/\\thmrdef\{([^\}]*)}/\<a id="$thmid"\>\<\/a\>/;
       $idtohtmlfile{$thmid} = $sechtmlfile{$chapsecnum};
       }

$thetheorem = qq|<\/p>\n<div class="$type">\n<p>\n<span class="theoremlabel">$Type |.$secnumber.".".$thmnumber." <\/span>".$thetheorem;

$thetheorem = $thetheorem.qq|\n<\/p>\n<\/div><!-- $type -->\n<p>|;

## DRG
## $thetheorem =~ s/\\thmrdef{([^\}]*)}//g;

if($theproof) {
   $theproof = qq|<div class="proof"><b>Proof.</b><br>\n<p>\n|.$theproof."\n</p>\n</div>\n\n";
}

return($thetheorem)
}

###############
## DRG New function to handle displays with 
## numbered equations.

sub processdisplay {
my $part1=shift;
my $chapsecnum=shift;
my $part2 = shift;
my $part3 = (shift || "");
my $equationid="";
my $thedisplay="DISPLAYERROR";
##  print "part2: ******************** $part2\n";

$chapsecnum =~ /([0-9]+).([0-9]+)/;
my ($chapnum,$secnum)=($1,$2);

$chapnum = ($chapnum || "cchapnum");
$secnum = ($secnum || "ssecnum");

$chapnum =~ s/^0*//;
$secnum =~ s/^0*//;
my $secnumber=$chapnum.".".$secnum;

my $equationnumber=globalcount("equation");

  $equationid=$part2;
  $equationid =~ s/[ \t\n]+/ /g;
  $xrefn{$equationid} = $secnumber.".".$equationnumber;
  $idtohtmlfile{$equationid} = $sechtmlfile{$chapsecnum};
  $thedisplay = "<a id=\"$equationid\"><\/a>\n\$\$".$part1."($xrefn{$equationid})".$part3."\$\$";

return($thedisplay)
}

################
sub fixcode {

# print "************************************** fixcode:@_\n";
my ($thecode) = @_;

# print "**************************************** thecode: $thecode\n";
$thecode =~ s/< /&lt;/sg;
$thecode =~ s/\\bs /\\/sg;
# print "**************************************** thecode: $thecode\n";

return($thecode)
}

################
sub processcolorlist {

# print "************************************** processcolorlist\n";
my $thelist = (shift || "");

 $thelist = qq|<div class="colorlist"><p>\n|.$thelist."\n</div>\n";

return($thelist)
}

################
sub processcode {

#print "************************************** processcode\n";
my $thecode = (shift || "");

 $thecode = fixcode($thecode);
 $thecode = qq|<pre><code><p>\n|.$thecode."\n</p></code></pre>\n";

return($thecode)
}

################
sub processproof {

my $theproof = (shift || "");

 $theproof = qq|<div class="proof"><p><b>Proof.</b><br>\n|.$theproof."\n</p>\n</div>\n\n";

return($theproof)
}

################
sub processdetachedproof {
  my $titlestring = shift;
  my $theproof = (shift || "");

  $theproof = qq|<div class="proof"><p><b>$titlestring</b><br>\n|.$theproof."\n</p>\n</div>\n\n";

  return($theproof)
}

################

sub processfigure {

my $chapsecnum=shift;
my $thetext = shift;
$chapsecnum =~ /([0-9]+).([0-9]+)/;
my ($chapnum,$secnum)=($1,$2);
my $fullchapnum = $chapnum;
#print "-------------->pre: $thetext\n";

$chapnum = ($chapnum || "cchapnum");
$secnum = ($secnum || "ssecnum");

$chapnum =~ s/^0*//;
$secnum =~ s/^0//;
my $secnumber=$chapnum.".".$secnum;

## DRG: Just one \htmlfigure command now.
$thetext =~ s/\\htmlfigure *{(.*?)} *\n/"<\/p><div class='figure'>".getfilecontents("html_figures",$1)/seg;
#print "-------------->post: $thetext\n";

## DRG
$thetext =~ /\\figrdef\{([^}]*)}/;
my $figureid=$1;

my $figcount=globalcount("figure");
if($figureid) {
      $figtitle{$figureid}=$figcount;
      $xrefn{$figureid}=$secnumber.".".$figcount;
      $figandcount =qq|<span class="figurenumber">Figure $xrefn{$figureid}.</span>|;
## DRG Replace figrdef by html anchor.
      $thetext =~ s/\\figrdef\{([^\}]*)}/\<a id="$figureid"\>\<\/a\>/;
      $idtohtmlfile{$figureid} = $sechtmlfile{$chapsecnum};
#      print "**************************************** id of $figureid: $idtohtmlfile{$figureid}\n";
      if (!defined( $idtohtmlfile{$figureid}) ) {
	$idtohtmlfile{$figureid} = $chaphtmlfile{$fullchapnum};
#        print "**************************************** chapnum: $fullchapnum\n";
#        print "**************************************** id of $figureid second try: $idtohtmlfile{$figureid}\n";
      }
}

#kill the whole caption but remember the caption text
$thetext =~ s/\\begincaption *\n(.*?)\n\\endcaption//s;
my $thecaption=$1;

# put the caption at the end of the figure

## DRG: tablelayout is gone.
## if($tablelayout) {
##  $thetext = "<table class='figure'>\n".$thetext."\n<tr><td class='caption' colspan='0'>".$figandcount." ". $thecaption."<\/td><\/tr>\n<\/table><!-- figure -->";
## }
## else{
#$thetext = $thetext."\n<div class='figcaption'>".$figandcount." ". $thecaption."<\/div>\n<\/div>\n";
$thetext = $thetext."<div class='figcaption'>".$figandcount." ". $thecaption."<\/div><\/div>\n<p>";
## }

return($thetext)

}


################

sub getfilecontents {

my $subdir=shift;
my $filename=shift;
## DRG: layout gone
## my $layout=shift;
print "In getfilecontents: $filename\n";

#  Here is the logic to how we group the images in a compound figure.
#
#  Suppose we have 7 images in a figure, and we want to arrange them
#  as  3 in the first row, 3 in the second row, and 1 in the third row.
#  In the htmlonly section of the figure environment we would see:
#
#  \htmlfigure{file_name_goes_herehtml,[3,3,1]}
#
#  That line is first read in the processfigure subroutine and then parsed in
#  the getfilecontents subroutine.  In getfilecontents the following
#  global variables are set up:
#
#  $layoutparameters=True;
#  $fignum=0 initially. It will count up to 6, keeping track of
#                      which image we are processing.
#  $rownum=0 initially. It will count up to 2, then up to 2 again,
#                  ad then up to 0.  It counts where we are in each row.
#  $therow=0 initially. It will count up to 2, keping track of the row we are on.
#  @rownum = (3,3,1).  A list of the number of images in each row.
#
#  Now all we do is wrap each row containing more than 1 image in a div
#  of class="figrow".
#
#  The remainder of this subroutine makes those global variables and then calls
#  the subroutine processonefigure on each of the figures.

## DRG: layout gone.
## if($layout) { $layoutparameters=1; $fignum=0; $rownum=0; $therow=0;
##   $layout =~ s/\[*\]*//g;
## print(" the layout: ", $layout, "\n");
##   @rownum = split(",",$layout);
## 
## print(" the rownum : ", @rownum ,"\n");
##   print("Layout parameters: ",$layout) }
## 
## else { $layoutparameters=0;
##     print("no layout parameters\n") }

## DRG: temporary trace
# print "Sucking in $inputdirectory/$subdir/$filename\n";
open THEFILE,  "<$inputdirectory/$subdir/$filename"  or warn "missing file: $! \n";
local $/=undef;
$thecontents=<THEFILE>;
close THEFILE;

#
#The file may contain several figures, each of which goes in a div
#
## $thecontents =~ s/(<div .*?<\/div>[ \t\n]*<script .*?<\/script>)/processonefigure($1)/seg;
# Okay, now that doesn't work because teh new format of the multiple image files
# has all the divs first, then all the scripts.  The previous iteration had divs 
# immediately followed by their scrpts, which is what the above expression assumes.

return($thecontents)
}

## DRG: No need for this any more. Figure internals are not processed.
################
## sub processonefigure {
## 
## my $thecontents = shift;
## 
## #
## # we use the layout parameters as global variables, but they should be passed to this function
## #
## 
## # print("PROCESSING one figure, rowum:",$rownum," \n");
## 
## # print $thecontents;
## 
## my $theAspectRatio;
## 
## $thecontents =~ /id *= *"([^"]+)"/;
## my $theid=$1;
##  print("\n the id : ",$theid ,"\n");
## 
## if($thecontents =~ /AspectRatio *=[ \(]*([0-9\.]+)/) {
##     $theAspectRatio=$1;
## print("Found aspect ratio \n")
## }
## elsif($thecontents =~ /aspect_ratio *= *([0-9\.]+)\/([0-9\.]+)/) {
##     $theAspectRatio=(1.0*$1)/$2
## }
## 
## else { $theAspectRatio=1.000001 }
## 
##  print("\nAspectRatio : ",$theAspectRatio ,"\n");
## 
## $thecontents =~ /width:([0-9]+)\%/;
## $thewidthpercent=$1;
## # print("The width (percent): ",$thewidthpercent,"\n");
## 
## $thecontents =~ s/class="jxgbox"/class="jxwhitman"/;
## $thecontents =~ s/float:left;//;
## $thecontents =~ s/float:left"/"/;
## 
## my $colspan0="";
## 
## my $thewidth=(470.0*$thewidthpercent)/80.0;
## # my $thewidth=(380.0*$thewidthpercent)/80.0;
## 
## my $theheight = $thewidth*$theAspectRatio;
## 
## my $thestyle="width:".$thewidth."px; height:".$theheight."px;";
## 
## #the following seems excessive, because it assumes that the only argument
## # to style is the width.  Maybe better to just substitute for the width percent?
## $thecontents =~ s/style *= *".*?\%;/style="$thestyle/g;
## $thecontents =~ s/style *= *".*?\%"/style="$thestyle"/g;
## 
## #
## #  Need to unify all the "maybe it has a caption" code
## #
## 
## if(!$layoutparameters) {
## # print(" NO layout parameters \n\n");
## 
##   # put the starting tag
##   if($tablelayout) {
##    #  $thecontents =~ s/(<div id=)/<tr class="figrow">\n$1/;
##      $thecontents =~ s/(<div id=)/<tr class="figrow">\n$1/;
##   }
##   else {
##      $thecontents =~ s/(<div id=)/<div class="figrow">\n$1/;
##   }
## 
##   # and the ending tag  (this one is untested in the non-table version)
##   if($tablelayout) {
##      $thecontents =~ s/(<\/div>)/$1\n<\/tr>\n/;
##   }
##   else {
##      $thecontents =~ s/(<\/div>)/$1<\/div>\n/;
##   }
## }
## else {
## 
## # print("checking which row: XX",$rownum,"YY",$therow,"ZZZ",$rownum[$therow],"AAA\n");
## 
##    if($rownum[$therow] == 1) {  #this means we have one image in the row, so we
##                                    # treat it just like the !$layoutparameters case
## 
## $colspan0=qq| colspan="0" |; # this figure should take up the whole row.  $colspan0 inserted below.
## 
## # put the starting tag
## if($tablelayout) {
##    $thecontents =~ s/(<div id=)/<tr class="figrow">\n$1/;
## }
## else {
##    $thecontents =~ s/(<div id=)/<div class="figrow">\n$1/;
## }
## 
## # and the ending tag  (this one is untested in the non-table version)
## if($tablelayout) {
##    $thecontents =~ s/(<\/div>)/$1<\/tr>\n/;
## }
## else {
##    $thecontents =~ s/(<\/div>)/$1<\/div>\n/;
## }
## 
##    ++$therow;
##    $rownum=0;
##    ++$fignum;
##   }
## 
##   elsif($rownum==0) {  #we are starting a row with more than one image, so start a div
## 
## if($tablelayout) {
##    $thecontents =~ s/(<div id=)/<tr class="figrow">\n$1/;
## }
## else {
##    $thecontents =~ s/(<div id=)/<div class="figrow">\n$1/;
## }
##                           # and then process the figure in the usual way
##    ++$rownum;
##    ++$fignum;
##     }
## 
##    elsif($rownum < $rownum[$therow] -1) { # in the middle of a row, so just
##                                              # process the figure in the usual way
##     ++$rownum;
##    ++$fignum;
##     }
## 
##    else {  # we must be at the end of the row, so end the row div
## print("LLLL\n At the end of a row of figures\n");
## if($tablelayout) {
##      $thecontents =~ s/(<\/div>)/$1\n<\/tr> <!-- figrow -->\n/;
## }
## else {
##      $thecontents =~ s/(<\/div>)/$1\n<\/div> <!-- figrow -->\n<div style="clear:left;"><\/div>\n /;
## }
##                            # and then process the figure in the usual way
##      ++$therow;
##    $rownum=0;
##    ++$fignum;
##     }
##       
## }
## 
## # having inserted the surrounding tags for the figure in each case, now we process
## # the actual figure.  Two cases, depending on whether there is a caption
## 
##   if($thecontents =~ /caption/) {
## if($tablelayout) {
##   $thecontents =~ s/(<div id="[^>]*)caption:'([^']*)'(.*?"><\/div>)/<td class="figure" $colspan0>\n$1$3\n<div class="figcaption">$2<\/div>\n<\/td><!-- figure -->\n/s;
## }
## else {
##   $thecontents =~ s/(<div id="[^>]*)caption:'([^']*)'("><\/div>)/<div class="figure">\n$1$3\n<div class="figcaption">$2<\/div>\n<\/div><!-- figure -->\n/s;
## }
##   }
##   else {
##   $thecontents =~ s/(<div id="[^>]*)("><\/div>)/<td class="figure" $colspan0 >\n$1$2\n<\/td><!-- figure -->\n/s;
##   }
## 
## return($thecontents)
## 
## }
## 
## 
################

###########################################################
## DRG: Added sage cell stuff, and a macro for superscripts.
sub header {

my $type=shift || "";
my $title=shift || "";
my $chapnum=shift || "";
my $secnum=shift || "";
my $exercise_section=shift || "";
my $alttitle=shift || "";
if (!($alttitle =~ /\S/)) {
  $alttitle = $title;
}
$myheader= qq@<!DOCTYPE HTML>
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<meta http-equiv="X-UA-Compatible" content="IE=Edge,chrome=1">
<meta http-equiv="Content-Language" Content="en">
<title>$alttitle</title>
<script type="text/javascript" src="js/deployJava.js"></script>
<!--
<link rel="stylesheet" type="text/css" href="http://jsxgraph.uni-bayreuth.de/distrib/jsxgraph.css" />
-->
<!-- use Jsxgraph to create figures: http://jsxgraph.uni-bayreuth.de/wp/ -->
<link rel="stylesheet" type="text/css" href="css/jsxgraph.css" />
<script type="text/javascript" src="js/jsxgraphcore.js"></script>
<script src="js/three.min.js"></script>
<script src="js/Detector.js"></script>
<script src="js/TrackballControls.js"></script>
<script src="js/OrthographicTrackballControls.js"></script>
<script src="js/THREEx.KeyboardState.js"></script>
<script src="js/THREEx.FullScreen.js"></script>
<script src="js/THREEx.WindowResize.js"></script>
<script type='text/javascript' src='js/DAT.GUI.min.js'></script>
<script>
window.requestAnimFrame = (function(){
    return  window.requestAnimationFrame       || 
        window.webkitRequestAnimationFrame || 
        window.mozRequestAnimationFrame    || 
        window.oRequestAnimationFrame      || 
        window.msRequestAnimationFrame     || 
        function(/* function */ callback, /* DOMElement */
            element){
            window.setTimeout(callback, 1000 / 60);
        };
})();
</script>
<script src="js/ifvisible.min.js"></script>
<script src="js/conrec.js"></script>
<script>
active = true;
ifvisible.setIdleDuration(10);
ifvisible.on("idle", function(){
    active = false;
//    console.log("idle");
});

ifvisible.on("wakeup", function(){
    active = true;
//    console.log("wakeup");
});
</script>
<script src="js/parser.js"></script>
<link href="css/whitman.css" rel="stylesheet" type="text/css" />@.
"<link href=\"css/$basename.css\" rel=\"stylesheet\" type=\"text/css\" />".
qq@<link href="css/knowlstyle.css" rel="stylesheet" type="text/css" />

<script type="text/x-mathjax-config">
MathJax.Hub.Config({
  extensions: ["tex2jax.js","TeX/AMSmath.js","TeX/AMSsymbols.js"],
  jax: ["input/TeX","output/HTML-CSS"],
  tex2jax: {
    inlineMath: [['\$','\$'],["\\\\(","\\\\)"]],
    processEscapes: true,
  },
TeX: {
   Macros: {
    val: "\\\\mathop{\\\\rm val}",
    divides: "\\\\mid",
    ndivides: "\\\\mathrel{\\\\not|}",
    fix: "\\\\mathop{\\\\rm fix}",
    id: "\\\\mathop{\\\\rm id}",
    na: "\\\\mathop{\\\\rm na}",
    d: "\\\\mathop{\\\\rm\\\\strut d}\\\\nolimits ",
    legendre: ["\\\\left(\\\\frac{#1}{#2}\\\\right)",2],
    normalbaselines: "",
    notdiv: "\\\\nmid",
    implies: "\\\\Rightarrow",
    iff: "\\\\Leftrightarrow",
    sevenpoint: "\\\\scriptsize",
    ds: "\\\\displaystyle",
    lcm: "\\\\operatorname{lcm}",
    arccot: "\\\\operatorname{arccot}",
    arcsec: "\\\\operatorname{arcsec}",
    arccsc: "\\\\operatorname{arccsc}",
    arccosh: "\\\\operatorname{arccosh}",
    arcsinh: "\\\\operatorname{arcsinh}",
    arcsech: "\\\\operatorname{arcsech}",
    sech: "\\\\operatorname{sech}",
    csch: "\\\\operatorname{csch}",
    R: "{\\\\mathbb R}",
    sb: "_",
    sp: "^",
    Q: "{\\\\mathbb Q}",
    C: "{\\\\mathbb C}",
    N: "{\\\\mathbb N}",
    Z: "{\\\\mathbb Z}",
    U: "{\\\\mathbb U}",
    v: ["\\\\langle #1\\\\rangle",1],
    dint: ["{\\\\mathchoice{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int}_{#1}}{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int}_{#1}}{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int}}{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int}}}",1],
    tint: ["{\\\\mathchoice{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int\\\\!\\\\!\\\\!\\\\int}_{#1\\\\;}}{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int\\\\!\\\\!\\\\!\\\\int}_{#1}}{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int\\\\!\\\\!\\\\!\\\\int}_{#1}}{\\\\mathop{\\\\int\\\\!\\\\!\\\\!\\\\int\\\\!\\\\!\\\\!\\\\int}_{#1}}}",1]
   }
},
  "HTML-CSS": { scale: 100},
  menuSettings: { zscale: "150\%", zoom: "Double-Click" }
});
    JXG.Options.text.display = 'html';
    JXG.Options.text.useMathJax = true;
</script>
<script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/math\
jax/2.7.1/MathJax.js?config=TeX-AMS-MML_HTMLorMML">
</script>
<script>
MathJax.Hub.Register.StartupHook("End",function () {
  if(typeof window.orientation === 'undefined'){
    var i,x;
    x = document.getElementsByClassName("webgl_display");
    for (i=0; i<x.length; i++) {
      x[i].style.display = "block";
    }
    x = document.getElementsByClassName("no_webgl_display");
    for (i=0; i<x.length; i++) {
      x[i].style.display = "none";
    }
  }
});
</script>

<script src="js/jquery-3.1.0.min.js"></script>
<script src="https://sagecell.sagemath.org/static/embedded_sagecell.js"></script>
<script src="js/jquery.visible.min.js"></script>
<script type="text/javascript" src="js/knowl.js"></script>
<style type="text/css">

.sagecell .CodeMirror {
  height: auto;
}

.sagecell .CodeMirror-scroll {
  overflow-y: hidden;
  overflow-x: auto;
}

.sagecell-practice .CodeMirror-scroll {
}

.sagecell button.sagecell_evalButton {
    font-size: 50%;
}

.sagecell_sessionContainer {
    margin-bottom:1em;
}
</style>
<script>
function toggle(a) {
  var b = String(a);
  b=b.replace("contents","button");
  if (document.getElementById(a).style.display == "block") {
    document.getElementById(a).style.display = "none";
    document.getElementById(b).src = "expand_submenu.png";
  } else {
    document.getElementById(a).style.display = "block";
    document.getElementById(b).src = "collapse_submenu.png";
  }
}
function toggle_sidebar() {
  if (document.getElementById('sidebar').style.display == "block") {
    document.getElementById('sidebar').style.display = "none";
    document.getElementById('menu_expand').style.display = "block";
  } else {
    document.getElementById('sidebar').style.display = "block";
    document.getElementById('menu_expand').style.display = "none";
  }
}
</script>

</head>

<body class=""@;
if ($chapnum =~ /\S/) {
  $myheader .= qq@ onload="toggle('chapter_@.$chapnum.qq@_contents')"@;
}
$myheader .= qq@>
<div id="header">
<div class="right">@;

$myheader .= breadcrumbs($type,$chapnum,$secnum);
$myheader .= search_box();
$myheader .=qq@<div id="title"><span id="title-content">$title</span></div>@;
if ($exercises_link && ($type eq "section") && $exercise_section) {
  print "*************************** section $chapnum $secnum has exercises\n";
  $myheader .=qq@<a href="#exercises">[Jump to exercises]</a>\n@;
}
$myheader .=qq@</div>
</div>
@;

return($myheader)

}

sub search_box {
open search_path_file, "$maininputdirectory/search_path.txt";
$search_path = <search_path_file>;
close search_path_file;
chomp $search_path;
#return(qq|<div id="search">
return(qq|
<FORM method=GET action=https://www.google.com/custom id="search">
<INPUT TYPE=text name=q size=31 maxlength=255 value="" style="font-size:small">
<INPUT type=submit name=sa VALUE="Search the book" style="font-size:small">
<INPUT type=hidden name=cof VALUE="AH:center;AWFID:198dc2b883340b1f;">
<input type=hidden name=domains value="www.whitman.edu">
<input type=hidden name=sitesearch value="|.$search_path.qq|">
</FORM>
|);
#</div>|);
}
##############################################
sub breadcrumbs {

my $type=shift || "";  
my $chap=shift || "";  
my $sect=shift || "";  

# print "-----------------------> breadcrumbs: type=$type, chap=$chap, sect=$sect\n";
my $chaptitle;
my $sectitle;
my $chapfile;
my $sectfile;

if($type eq "chapter") { $chaptitle=$chaptitle{$chap}; $chapfile="chapter".$chap.".html"}
elsif($type eq "section") {$chaptitle=$chaptitle{$chap};  $sectitle = $sectitle{$sect};
                           $chapfile="chapter".$chap.".html"; $sectfile="section".$sect.".html"}
elsif($type eq "frontmatter") { $chaptitle = $chaptitle{$chap}; $chapfile=$chap.".html"}

$chapfile=($chapfile || "error");
$sectfile=($sectfile || "error");
$chaptitle = ($chaptitle || "The Chapter Title");
$sectitle = ($sectitle || "The Section Title");

my $breadcrumbs=
qq|\n<div class="bread">
  <a href=".">Home</a>|;
if ($chap =~ /\S/) {
  $breadcrumbs .= qq| &raquo; <a href="$chaphtmlfile{$chap}">$chaptitle{$chap}</a>|;
}

if($sect) {
$breadcrumbs .= qq| &raquo; <a href="$sechtmlfile{$sect}">$sectitle{$sect}</a>|;
}

$breadcrumbs .= qq|\n</div>\n|;

return($breadcrumbs)

}
###############################################

sub footer {

$myfooter = qq|</div></div></body>\n</html>\n|;

return($myfooter);

}

###############################################

sub titlepage {

local $titlepage=qq|
<div id="main">
|.nextsectionnav("00.00").qq|
<div id="content">
<p>
|;

open THEFILE,  "<$maininputdirectory/front_page.html"  or warn "missing file: $! \n";
local $/=undef;
$titlepage.=<THEFILE>;
close THEFILE;

return($titlepage)

}

###############################################

## DRG: Various changes here to make figures work.
## Also made previous/next nav fixed on browser window.
## Now a static file, this is out of date.
##sub writecssfile {
##
##open CSSFILE, ">$outdir/whitman.css";
##
##print CSSFILE <<CSS;
##
##div.jxgbox {
##   margin-left: auto;
##  margin-right:auto;
##}
##
##div #content { max-width: 600px;
##              font-size: 100%;
##              line-height:125%;
##     }
##
##* p { max-width: 600px }
##div.definition { max-width: 600px }
##div.definition span.theoremlabel { 
##     font-style: normal;
##     font-weight: bold;
##     margin-right: 10px;
##   }
##
##div.remark { max-width: 600px }
##div.remark span.theoremlabel {
##     font-style: normal;
##     font-weight: bold;
##     margin-right: 10px;
##   }
##
##div.example { max-width: 600px }
##div.example span.theoremlabel {
##     font-style: normal;
##     font-weight: bold;
##     margin-right: 10px;
##   }
##
##div.theorem { max-width: 600px;}
##div.theorem p { font-style: italic; }
##div.theorem span.theoremlabel { 
##     font-style: normal;
##     font-weight: bold;
##     margin-right: 10px;
##   }
## 
## #next { 
##     top:10px;
##     right: 10px;
##     text-align:right;
##     position:fixed
## /* original:
##         margin-right: 10px;
##         text-align:right;
##         float:right;
## */
##       }
## 
## div.figure {
##    margin-top: 20px;
##     max-width: 600px
## }
## 
## div.figcaption {
##    text-align: center;
##         width: 80%;
##   margin-left: auto;
##  margin-right: auto;
##    margin-top: 7px;
##     font-size: 90%;
##    font-style: italic;
## }
## 
## div.qed {
##     margin: -10pt 0 20pt 0;
## }
## 
## span.figurenumber {
##     font-weight: bold;
## }
## 
## CSS
## 
## close(CSSFILE);
## 
## }
## 
#####################

sub chapsecnumrewrite {

my $chaporsec=shift;

$chaporsec = ($chaporsec || "");

$chaporsec =~ s/^0*//;
$chaporsec =~ s/\.0*/\./;

return($chaporsec);
}

#####################

sub xrefn {   # we can't accomplish this using only a hash because the TeX 
              # source can have line breaks inside \xrefn.  So we need to
              # convert the reference key to a standard form

my $ref=shift;
$ref =~ s/[ \t\n]+/ /g;

return($xrefn{$ref} || "MISSING XREFN(".$ref.")")
}
#####################

sub normalizeref {   # we can't accomplish this using only a hash because the TeX source can have
              # line breaks inside \xrefn.  So we need to convert the reference key to a standard form

my $ref=shift;

$ref =~ s/[ \t\n]+/ /g;

return($ref);
}
###################
## DRG: New nav links:
## Each chapter file points to the first section in "next", the 
##   last section of the previous chapter in "previous".
## Last section of a chapter points to next chapter in "next".
## Section 1 points to chapter file in "previous".
## Home "next" points to chapter 1.
sub nextsectionnav {
my $secnum=shift;
##print "************************** secnum: $secnum\n";

$secnum=~ /([0-9]{2})\.([0-9]{2})/;
my ($thechap,$thesec)=($1,$2);
##print "************************** thechap: $thechap\n";
##print "************************** thesec: $thesec\n";

my $nextsection = $thechap.".".nextnum($thesec,1);

my $theprevsec=nextnum($thesec,-1);
##print "******************************* 1: $theprevsec\n";
my $thenextdiv = qq|\n<div id="next">|;
##print "******************************* $chaphtmlfile{$thechap}\n";
##print "******************************* $theprevsec\n";
## DRG "previous" from section 1 points to the chapter file.
if($theprevsec < 0) {
##  print "******************************* $theprevsec\n";
##  print "******************************* sec: $thesec\n";
##  print "******************************* chap: $thechap\n";
  my $theprevchap = nextnum($thechap,-1);
  my $testsec = "";
  my $prevtestsec;
#  if ($theprevchap < 10) { $theprevchap = "0".$theprevchap; }
  if ($theprevchap > 0) {
    my $section = 0;
    do {
      $section=nextnum($section,1);
      $prevtestsec = $testsec;
      $testsec = $theprevchap.".".$section;
##      print "*********************** testing: $testsec\n";
    } until !$sechtmlfile{$testsec};
##    print "************************* using $prevtestsec\n";
##    $thenextdiv = $thenextdiv.qq|<a href="|.$sechtmlfile{$prevtestsec}.qq|">previous</a><br/>|;
    if ($section == 1) {
      $thenextdiv = $thenextdiv.qq|<a href="|.$chaphtmlfile{$theprevchap}.qq|"><img src="previous_section.png" width="30"></a>|;
    } else {
      $thenextdiv = $thenextdiv.qq|<a href="|.$sechtmlfile{$prevtestsec}.qq|"><img src="previous_section.png" width="30"></a>|;
    }
  }
}
elsif($theprevsec > 0) {  
  $theprevsec = $thechap.".".$theprevsec;
  $thenextdiv = $thenextdiv.qq|<a href="|.$sechtmlfile{$theprevsec}.qq|"><img src=previous_section.png width="30"></a>| 
} elsif($chaphtmlfile{$thechap} =~ /\S/) {
  $thenextdiv = $thenextdiv.qq|<a href="|.$chaphtmlfile{$thechap}.qq|"><img src=previous_section.png width="30"></a>| 
}

# need to fix this so it properly handles the last section of a chapter.
## DRG I made the last section point to the next chapter.
if($sechtmlfile{$nextsection}) {
   $thenextdiv =$thenextdiv.qq|&nbsp;&nbsp;<a href="|.$sechtmlfile{$nextsection}.qq|"><img src=next_section.png width="30"></a></div>\n|;
}
else {
  my $nextchap = $thechap+1;
  if ($nextchap < 10) { $nextchap = "0".$nextchap; }
  if ($chaphtmlfile{$nextchap}) {
    $thenextdiv =$thenextdiv.qq|&nbsp;&nbsp;<a href="|.$chaphtmlfile{$nextchap}.qq|"><img src=next_section.png width="30"></a></div>\n|;
  } else {
   $thenextdiv =$thenextdiv.qq|</div>\n|;
  }
}
return($thenextdiv)

}

###################

sub nextnum {

my $num=shift;
my $offset=shift;
##print "******************* num: $num\n";
##print "******************* offset: $offset\n";
## $num =~ s/^0//;

$num=$num+$offset;

# print("offset ",$offset,"    ",$offset+$offset,"   ",$num,"\n");

#if($num eq 0) { return("") }
#if($num <= 0) { return("") }

if($num >= 0 && $num<10) { $num = "0".$num }

return("".$num)

}
###################

sub usage {
    my $commandname = $0;
    $commandname =~ s/^.*\///;
    print "$commandname [options] inputdirectory outputdirectory\n".
        "  -e don't include \"jump to exercises link\"\n".
        '  -s make single variable version'."\n".
        '  -l make late transcendentals version'."\n";
  }
